package component.system;

public class EDFMessageQueueHeap {
	
	private static int INITIAL_CAPACITY		= 100;
	private static int CAPACITY_INCREMENT	= 100;
	
	private MessageEntry [] messages		= null;
	private int count						= 0;  
	
	public EDFMessageQueueHeap() {
		messages = new MessageEntry[INITIAL_CAPACITY];
		count = 0;
    }

	/*
		This method has a worst case completion time of O(log(n))
	*/    
    public synchronized void schedule(long deadline, Message message) {
    	if (count == (messages.length - 1)) increment();
    	messages[++count] = new MessageEntry(deadline,message);
    	// start of rebalancing procedure after insertion
    	int parentIndex = count/2;
    	int sonIndex = count;
    	// this has a worst case of O(log(n))
    	while (parentIndex != 0)
    	{
    		if (messages[parentIndex].deadline <= deadline) break;
			MessageEntry temp = messages[sonIndex];
			messages[sonIndex] = messages[parentIndex];
			messages[parentIndex] = temp;
			parentIndex = parentIndex / 2;
		}	    	
	}
	
	/*
		This method has a worst case completion time of O(log(n))
	*/
	public synchronized Message getNextMessage() {
		if (count == 0) return null;
		if (count == 1) {
			count = 0;
			Message result = messages[1].message;
			messages[1] = null;
			return result;
		}
		Message result = messages[1].message;
		messages[1] = messages[count];
		messages[count] = null;
		count--;
		int sonIndex = 2;
		// this has a worst case of O(log(n))
		while (sonIndex <= count) {
			boolean hasRightBrother = ((sonIndex + 1) <= count);
			if (hasRightBrother) {
				boolean rightBrotherFirst = (messages[sonIndex].deadline > messages[sonIndex+1].deadline);
				if (rightBrotherFirst) {
					if (messages[sonIndex+1].deadline >= messages[sonIndex/2].deadline) break;
					MessageEntry temp = messages[sonIndex/2];
					messages[sonIndex/2] = messages[sonIndex+1];
					messages[sonIndex+1] = temp;
					sonIndex = 2 * (sonIndex + 1);
				}
				else {
					if (messages[sonIndex].deadline >= messages[sonIndex/2].deadline) break;
					MessageEntry temp = messages[sonIndex/2];
					messages[sonIndex/2] = messages[sonIndex];
					messages[sonIndex] = temp;
					sonIndex = 2 * sonIndex;
				}
			}
			else {
				if (messages[sonIndex].deadline >= messages[sonIndex/2].deadline) break;
				MessageEntry temp = messages[sonIndex/2];
				messages[sonIndex/2] = messages[sonIndex];
				messages[sonIndex] = temp;
				sonIndex = 2 * sonIndex;
			}
		}
		return result;
	}
	
	private synchronized void increment() {
		MessageEntry [] newMessages = new MessageEntry[messages.length + CAPACITY_INCREMENT];
		for (int i = 1; i <= count; i++) {
				newMessages[i] = messages[i];
		}
		messages = newMessages;
	}	
}    
