package component.system;

public class EDFMessageQueueSimple {
    /*
    	messages: a linked list in which messages are sorted based on deadline
    */
    private MessageEntry messagesBegin;
    private MessageEntry messagesEnd;

    /*
    	Consctructor of class EDFMessageQueueSimple.  
    */
    public EDFMessageQueueSimple(){
    	messagesBegin = null;
    	messagesEnd   = null;
    }

    /*
    	Method used by EDFScheduler to put a message in a queue.  Two parameters
        are given: the absolute deadline of the message and the message itself.
        If the absolute deadline is lower than 0, this means the deadline has
        no importance.  As a result the message is stored as last entry in the
        list.
        TODO: clean code.  At this time, starvation of non-deadline-constrained
        messages is not possible. If there is a message with deadline that cannot
        be put before a non-deadline message; then it will be put after it (since
        the deadline of a non-deadline message is lower than 0.
    */
    public synchronized void schedule(long deadline, Message message) {
    	MessageEntry messageEntry = new MessageEntry(deadline,message);
    	// test if the messages list is empty
    	if (messagesBegin == null) {
    		messagesBegin = messageEntry;
    		messagesEnd   = messageEntry;
    		messageEntry.next = null;
    	}
    	else {
    		// check if there is no deadline
    		if (deadline < 0) {
    			messagesEnd.next = messageEntry;
    			messageEntry.next = null;
    			messagesEnd = messageEntry;
    		}	
    		else {
    			if (messagesBegin.deadline > deadline) {
    				messageEntry.next = messagesBegin;
    				messagesBegin = messageEntry;
    			}
    			else {
    				MessageEntry current = messagesBegin;
    				MessageEntry previous = messagesBegin;
    				while ((current != null) && (current.deadline <= deadline)) {
    					previous = current;
    					current = current.next;
    				}
    				previous.next = messageEntry;
    				messageEntry.next = current;
    				if (current==null) messagesEnd = messageEntry;
    			}
    		}	
    	}
    }

    /*
		This method is used by the PriorityExecutionLoop to retreive messages 
    */
    public synchronized Message getNextMessage() {
    	if (messagesBegin!=null) {
    		MessageEntry nextMessageEntry = messagesBegin;
    		messagesBegin = messagesBegin.next;
    		if (messagesBegin==null) messagesEnd = null;
    		return nextMessageEntry.message;
    	}
    	else return null;
    	// TODO: add notify code for PriorityExecutionLoop	
  	}
}


