package component.system;

// Earliest Deadline First scheduler
// TO DO: use a more efficient data representation structure for storing messages (tree)
public class EDFScheduler implements Scheduler{
    private EDFMessageQueueSimple[] EDFQueues;
    private PriorityExecutionLoop[] executionLoops;
    private int priorityLevels;
    private int minPriority;

    // This constructor will create minPriority + maxPriority + 1 message queues
    // with priorities minPriority, minPriority + 1, ... and maxPriority
    // PRECOND: Thread.MIN_PRIORITY <= minPriority <= maxPriority <= Thread.MAX_PRIORITY
    // TO DO: addition of a constructor that creates threads of equal priority
    public EDFScheduler(int minPriority, int maxPriority) {
    	this.priorityLevels = minPriority + maxPriority + 1;
    	this.minPriority = minPriority;
    	EDFQueues = new EDFMessageQueueSimple[priorityLevels];
        for (int i = 0; i < priorityLevels; i++) {
            EDFQueues[i] = new EDFMessageQueueSimple();
        }
    }
    
    // creation of threads, each with a distinct priority 
    public void start() {
        executionLoops = new PriorityExecutionLoop[priorityLevels];
        for (int i = 0; i < priorityLevels; i++) {
        	executionLoops[i] = new PriorityExecutionLoop(EDFQueues[i], minPriority + i);
        }
    }
    
    public void scheduleForExecution(Message message) {
    	int priority = minPriority;
    	long absoluteDeadline = -1;
    	
    	Integer prio = (Integer)(message.getHiddenField("TIMING.PRIORITY"));
    	Long maxET = (Long)(message.getHiddenField("TIMING.MAXENDTIME"));
    	Long subT = (Long)(message.getHiddenField("TIMING.SUBMITTIME"));
    	
    	// if a priority is given: set the priority
    	if (prio!=null) {
    		priority = prio.intValue();
        } 
        
        // if maxEndTime and submitTime are given: calculate the absoluteDeadline
    	if ((maxET!=null) && (subT!=null)) {
    		absoluteDeadline = maxET.longValue() + subT.longValue();
        }
        
        // TODO: hardcode timing parameters in Message
        // TODO: check priority range
        
    	// if no priority is given, the message will be scheduled in the
    	// queue with the lowest priority
        EDFQueues[priority].schedule(absoluteDeadline,message);
    }
}


