package component.system;


/**
  * This interface has to be implemented by every class that wants to
  * observe events at a Port.  Following events can be observed:
  *
  * - a message has been sent
  * - a message has been received
  * - a message has been processed
  */
public interface PortObserver {
	/**
	  * The port calls this message when a message is sent out over
	  * the port.
	  */
	public void messageSent(Message message);
	
	/**
	  * The port calls this message when a message is received at the
	  * port
	  */
	public void messageReceived(Message message);
	
	/**
	  * The port calls this message when a message is processed at the
	  * Port
	  */
	public void messageProcessed(Message message);
}