package component.examples.playfield;
import component.system.*;
import java.lang.reflect.*;
import java.util.*;

class Lock
  extends Global
{
   Object      statebefore;
   LockAble      state;
   Transaction lockedby;
   int         lockcount ;
   static private TreeMap locks = new TreeMap();

  /**
   * This method can be used to get a lock on an 
   * object. Depending on whether it was locked or
   * not will the transaction id be different.
   */
   static public Lock getLock(LockAble o)
     {
	if (o==null)
	  Fatal("lock: getlock; o = null");
	Lock lock=(Lock)locks.get(o);
	if (lock==null)
	  {
	     return new Lock(o);
	  }
	else
	  {
	     return lock;
	  }
     }
   
   protected Lock(LockAble tolock)
     {
	state=tolock;
	statebefore=tolock.getState();
	locks.put(tolock,this);
     }
   
   /**
    * Use the lock method to assign a certain
    * lock to a certain transaction.
    */
   protected void lock(Transaction by)
     {
	if (lockedby!=by && lockedby!=null)
	  Fatal("Lock: -1-");
	lockedby=by;
     }
   
   /**
    * The unlock method can be used to unlock a certain 
    * object. In this case the objects state is thrown away
    * and we remove ourselve from the locks
    */
  public void unlock()
    {
    locks.remove(state);
    state=null;
    }
  public void abortlock()
    {
    state.setState(statebefore);
    unlock();
    }
}

public class Transaction
{
   public  Integer id;
   static private int     nextid = 1;
   static private TreeMap transactions = new TreeMap();
   public  TreeSet locks;
   public Transaction()
     {
	id = new Integer(nextid++);
	locks = new TreeSet();
	transactions.put(id,this);
     }
   
   static public Transaction get(Integer id)
     {
	Transaction t=(Transaction)transactions.get(id);
	return t;
     }

   public boolean lock(LockAble obj)
    {
    Lock l=Lock.getLock(obj);
    if (l.lockedby==null) 
      {
	 l.lock(lockedby);
	 locks.add(l);
      }
    if (l.lockedby==this) 
      return true;
    return false;
    }
    
  /**
   * commit commits the transaction and 
   * removes itself.
   */
  public void commit()
    {
    Iterator it=locks.iterator();
    while(it.hasNext())
      {
      Lock lock=(Lock)it.next();
      lock.unlock();
      }
    transactions.remove(id);
    }

  /**
   * abort aborts the transaction and 
   * removes itself.
   */
  public void abort()
    {
    Iterator it=locks.iterator();
    while(it.hasNext())
      {
      Lock lock=(Lock)it.next();
      lock.abortlock();
      }
    transactions.remove(id);
    }
}

