package component.parser;
import component.productions.*;

public class ASTMethodDeclaration 
  extends SimpleNode 
{
   public ASTMethodDeclaration(int id) 
     {
	super(id);
     }
   static public boolean messageused = false;
   protected void outputModifiers()
     {
	int i=0;
	if (fields==null) output(" ");
	else while(i<fields.length)
	  {
	     String s= (String)fields[i];
	     output(s+" ");
	     i++;
	  }
     }
   protected boolean validMessageModifiers()
     {
	int i=0;
	if (fields==null) return true;
	while(i<fields.length)
	  {
	     String s= (String)fields[i];
	     if (s.compareTo("abstract")!=0) return false;
	     i++;
	  }
	return true;
     }
   public void generateSource()
     {
	/* sanity checks 
	 * 1. messages should have no modifiers
	 * 2. messages have no parameterlist (at the moment)
	 * 3. messages don't throw anything
	 * 4. message declarators should start with a capital
	 */
	int i;
	boolean haveblock=false;
	boolean ismessage=false;
	for(i=0;i<children.length;i++)
	  {
	     SimpleNode c=(SimpleNode)children[i];
	     if (c instanceof ASTResultType) 
	       {
		  ismessage=(c.fields!=null) && ("message".compareTo((String)c.fields[0])==0);
		  if (ismessage)
		    {
		       if (!validMessageModifiers()) /* (1) */
			 {
			    warning("messages shouldn't have any standard modifiers");
			    fields=null;
			 }
		       output("public synchronized void");
		       messageused=true;
		    }
	       }
	     if (ismessage && (c instanceof ASTMethodDeclarator))
	       {
		  char t=((String)(c.fields[0])).charAt(0);
		  if (t<'A' || t>'Z')
		    warning("message names should start with a capital");
	       }
	     if (c instanceof ASTNameList)
	       {
		  if (ismessage) /* (3) */
		    warning("messages shouldn't throw anything");
	       };
	  };
	if (ismessage)   /* outputting a message */
	  {
	     outputModifiers();
	     for(i=0;i<children.length;i++)
	       {
		  SimpleNode c=(SimpleNode)children[i];
		  if (c instanceof ASTMethodDeclarator)
		    {
		       output(c.fields[0]);
		       output("(Port port, Message inmessage, Message outmessage)");
		    }
		  if (c instanceof ASTBlock)
		    {
		       output("\n");
		       c.generateSource();
		       haveblock=true;
		    }
		  output(" ");
	       };
	     if (!haveblock) output(";");
	  }
	else
	  {
	     outputModifiers();
	     for(i=0;i<children.length;i++)
	       {
		  SimpleNode c=(SimpleNode)children[i];
		  if (c instanceof ASTResultType)
		    c.generateSource();
		  if (c instanceof ASTMethodDeclarator)
		    c.generateSource();
		  if (c instanceof ASTNameList)
		    {
		       output("throws ");
		       c.generateSource();
		    };
		  if (c instanceof ASTBlock) 
		    {
		       output("\n"); 
		       c.generateSource(); 
		       haveblock=true;
		    }
		  output(" ");
	       };
	     if (!haveblock) output(";");
	  }
     }
   public Production generateProductions()
     {
	int i;
	Production leftarrow=null, rightarrow=null;
	for(i=0;i<children.length;i++)
	  {
	     SimpleNode s=(SimpleNode)children[i];
	     if (s instanceof ASTMethodDeclarator) leftarrow=s.generateProductions();
	     if (s instanceof ASTBlock) rightarrow=s.generateProductions();
	  }
	setCurrentMethodName(null);
	return new ArrowProduction(leftarrow,rightarrow);
     }
}
