package component.parser;
import java.io.*;
import component.productions.*;

public class SimpleNode implements Node 
{
   protected Node parent;
   protected Node[] children;
   protected Object[] fields;
   protected int id;
   public SimpleNode(int i) {id = i;}
   public void jjtOpen() {}
   public void jjtClose() {}
   public void jjtSetParent(Node n) { parent = n; }
   public Node jjtGetParent() { return parent; }
   public void jjtAddChild(Node n, int i) {
    if (children == null) {
       children = new Node[i + 1];
    } else if (i >= children.length) {
       Node c[] = new Node[i + 1];
       System.arraycopy(children, 0, c, 0, children.length);
       children = c;
    }
      children[i] = n;
   }
   
   public void jjtSetField(Object n, int i) 
     {
	if (fields == null) 
	  {
	     fields = new Object[i + 1];
	  } 
	else if (i >= fields.length) 
	  {
	     Object c[] = new Object[i + 1];
	     System.arraycopy(fields, 0, c, 0, fields.length);
	     fields = c;
	  }
	fields[i] = n;
     }
   
   public void jjtAddField(Object n) 
     {
	if (fields==null) jjtSetField(n,0);
	else jjtSetField(n,fields.length);
     }
   
   public Node jjtGetChild(int i) 
     {
	return children[i];
     }
   public int jjtGetNumChildren() 
     {
	return (children == null) ? 0 : children.length;
     }
   
   /* You can override these two methods in subclasses of SimpleNode to
    customize the way the node appears when the tree is dumped.  If
    your output uses more than one line you should override
    toString(String), otherwise overriding toString() is probably all
    you need to do. */
   
   public String toString() { return TransformerTreeConstants.jjtNodeName[id]; }
   public String toString(String prefix) { return prefix + toString(); }
   
   /* output data to a certain file */
   private static PrintStream outputstream=null;
   protected static void setOutputStream(OutputStream os)
     {outputstream=new PrintStream(os);}
   protected static void output(Object str)
     {outputstream.print(str);}
   
   protected void outputFields(Object seperator)
     {
	int i;
	if (fields==null) return;
	for(i=0;i<fields.length;i++)
	  {
	     output(fields[i]);
	     if (i<fields.length-1) output(seperator);
	  }
     }
   protected void outputFields(Object seperator, Object terminator)
     {
	outputFields(seperator);
	output(terminator);
     }
   protected void outputFields(Object prefix, Object seperator, Object terminator)
     {
	output(prefix);
	outputFields(seperator,terminator);
     }
   
   protected void generateChildSources()
     {
	generateChildSources("");
     }
   protected void generateChildSources(Object seperator, Object suffix)
     {
	generateChildSources(seperator);
	output(suffix);
     }
   protected void generateChildSources(Object prefix, Object seperator, Object suffix)
     {
	output(prefix);
	generateChildSources(seperator,suffix);
     }
   protected void generateChildSources(Object seperator)
     {
	if (children != null) {
	   for (int i = 0; i < children.length; ++i)
	     {
		SimpleNode n = (SimpleNode)children[i];
		if (n != null) 
		  {
		     n.generateSource();
		     if (i<children.length-1) 
		       output(seperator);
		  }
	     }
	}
     }

  protected void interleave()
  { int i;
	for(i=0;i<children.length;i++)
    {
        ((SimpleNode)children[i]).generateSource();
        if (fields!=null && i<fields.length) output(fields[i]);
    }}

  protected void fieldsAndChilds()
  {
    outputFields("");
    generateChildSources();
  }

  protected void childsAndFields()
  {
    generateChildSources();
    outputFields("");
  }

   protected void assertion(boolean value)
     {
	if (!value)
	  {System.out.println("assertion failed...");
	     throw new RuntimeException();}
     }
   
   protected void remangleAsyncOperator()
     {
	ASTArgumentList s;
	String invocation;
	// the method to invoke is stored
	invocation=retrieveInvoke();
	output("{ outmessage.putField(\"Invoke\",\""+invocation+"\");\n");
	// send the message with sendMessage
	s=retrieveArgumentList();
	if (s!=null)
	  s.generateChildSources(";\n",";\n");
	// whatever the left hand side is it becomes the new target...
	output("(");
	((SimpleNode)children[0]).generateSource();
	output(").sendMessage(outmessage);\n");
	output("outmessage=new Message(inmessage);}\n");
     }
   
   private SimpleNode follow(SimpleNode n, int numfields, int numchilds, int getchildnum)
     {
	String err=null;
	// check size of fields:
	if (n.fields==null)
	  {
	     if (numfields!=0) err="wrong field count (0)";
	  }
	else if (n.fields.length!=numfields) err="wrong field count";
	// check size of children;
	if (n.children.length!=numchilds) err="wrong numchilds count";
	// return correct child if no errors
	if (err!=null)
	  {
	     System.out.println(err);
	     System.out.println(n);
	     System.exit(50);
	  }
	return (SimpleNode)n.children[getchildnum];}
   protected GetFieldProduction retrieveGetField(SimpleNode current)
     {
	Production res=null;
	try
	  {
	     // ShiftExpression
	     if (!(current instanceof ASTShiftExpression)) return null;
	     if (current.children.length!=1) return null;
	     current=(SimpleNode)current.children[0];
	     // AdditiveExpression
	     if (!(current instanceof ASTAdditiveExpression)) return null;
	     if (current.children.length!=1) return null;
	     current=(SimpleNode)current.children[0];	     
	     // MultiplicativeExpression
	     if (!(current instanceof ASTMultiplicativeExpression)) return null;
	     if (current.children.length!=1) return null;
	     current=(SimpleNode)current.children[0];
	     // UnaryExpression
	     if (!(current instanceof ASTUnaryExpression)) return null;
	     if (current.children.length!=1) return null;
	     current=(SimpleNode)current.children[0];	     
	     // UnaryExpressionNotPlusMinus
	     if (!(current instanceof ASTUnaryExpressionNotPlusMinus)) return null;
	     if (current.children.length!=1) return null;
	     current=(SimpleNode)current.children[0];	     
	     // PostFixExpression
	     if (!(current instanceof ASTPostfixExpression)) return null;
	     if (current.children.length!=1) return null;
	     current=(SimpleNode)current.children[0];	     
	     // PrimaryExpression
	     if (!(current instanceof ASTPrimaryExpression)) return null;
	     if (current.children.length!=1) return null;
	     current=(SimpleNode)current.children[0];	     
	     // PrimaryPrefix
	     if (!(current instanceof ASTPrimaryPrefix)) return null;
	     if (current.children.length!=1) return null;
	     current=(SimpleNode)current.children[0];
	     res=current.generateProductions();
	     if (res instanceof GetFieldProduction) return (GetFieldProduction)res;
	     return null;
	  }
	catch(Exception e)
	  {
	     System.out.println(e);
	     e.printStackTrace();
	     System.exit(0);
	  }
	System.out.println("current node = "+current+"/"+current.children.length);
	return null;
     }
   private ASTPrimaryExpression retrievePrimaryExp()
     {  // loop alle kinderen af in de diepte zolang ze niet meer
	// dan 1 expressie bevatten. We stoppen bij een
	// hiertoe hebben we een functie follow die een tree pakt controleert op
	// een aantal zaken en een nieuwe node weergeeft.
	try{
	   ASTExpression e=(ASTExpression)children[2];
	   ASTConditionalExpression ce=(ASTConditionalExpression)follow(e,0,1,0);
	   ASTConditionalOrExpression coe=(ASTConditionalOrExpression)follow(ce,0,1,0);
	   ASTConditionalAndExpression cae=(ASTConditionalAndExpression)follow(coe,0,1,0);
	   ASTInclusiveOrExpression ioe=(ASTInclusiveOrExpression)follow(cae,0,1,0);
	   ASTExclusiveOrExpression eoe=(ASTExclusiveOrExpression)follow(ioe,0,1,0);
	   ASTAndExpression ae=(ASTAndExpression)follow(eoe,0,1,0);
	   ASTEqualityExpression ee=(ASTEqualityExpression)follow(ae,0,1,0);
	   ASTInstanceOfExpression ie=(ASTInstanceOfExpression)follow(ee,0,1,0);
	   ASTRelationalExpression re=(ASTRelationalExpression)follow(ie,0,1,0);
	   ASTShiftExpression se=(ASTShiftExpression)follow(re,0,1,0);
	   ASTAdditiveExpression ame=(ASTAdditiveExpression)follow(se,0,1,0);
	   SimpleNode current=follow(ame,0,1,0);				 //multiplicativeexpression
	   current=follow(current,0,1,0);				 //unaryexpression
	   current=follow(current,0,1,0);				 //unaryexpressionnotplusminus
	   if(!(current instanceof ASTUnaryExpressionNotPlusMinus))
	     {  System.out.println("wrong expression type. Expected unaryexpresionnotplusminus");
		System.out.println(current);
		System.exit(0);}
	   current=follow(current,0,1,0);				 //postfixexpression
	   if(!(current instanceof ASTPostfixExpression))
	     {  System.out.println("wrong expression type. Expected postfixexpression");
		System.out.println(current);
		System.exit(0);}
	   current=follow(current,0,1,0);				 	 //primaryexpression
	   return (ASTPrimaryExpression)current;}
	catch(Exception e)
	  {
	     System.out.println(e);
	     System.exit(0);
	  }
	return null;
     }
   private String retrieveInvoke()
     {ASTPrimaryExpression EXP=retrievePrimaryExp();
  	SimpleNode Invoke, Args;
    Invoke=follow(EXP,0,2,0);						//primaryprefix
    Invoke=follow(Invoke,0,1,0);					//Name
    if (!(Invoke instanceof ASTName))
    	{System.out.println("was expecting a name");
		System.exit(80);}
    return (String)Invoke.fields[0];}
   private ASTArgumentList retrieveArgumentList()
     {
	ASTPrimaryExpression EXP=retrievePrimaryExp();
  	SimpleNode Invoke, Args;
	Invoke=follow(EXP,0,2,1);						//primarysuffix
	Invoke=follow(Invoke,0,1,0);					//Arguments
	if (!(Invoke instanceof ASTArguments))
	  {
	     System.out.println("was expecting arguments");
	     System.exit(80);
	  }
	if (Invoke.children==null) return null;
	Invoke=follow(Invoke,0,1,0);					//ArgumentList
	return (ASTArgumentList)Invoke;
     }
   

  /* generates java source from this AST */
   public void generateSource()
     {
	fatal(null);
     }
   
   public void fatal(String text)
     {
	System.out.println("  --Err: >"+this+"<");
	if (text!=null) System.out.println(text);
	System.out.println("  --Err: please send this message to");
	System.out.println("  --Err: werner.van.belle@vub.ac.be");
	System.exit(20);
     }

   public void warning(String text)
     {
	System.out.println("  --Warning: "+text);
     }
   
   /* returns a production for this AST */
   public Production generateProductions()
     {
	warning("missing "+this+" productiongenerator.");
	System.exit(0);
  	return null;
     }
   protected Production generateProductions(int i)
     {SimpleNode s=(SimpleNode)children[i];
	return s.generateProductions();}
   private static String currentClassName=null;
   protected static void setCurrentClassName(Object cn)
     {currentClassName=(String)cn;}
   protected static String getCurrentClassName()
     {return currentClassName;}
   private static String currentMethodName=null;
   protected static void setCurrentMethodName(Object cn)
     {currentMethodName=(String)cn;}
   protected static String getCurrentMethodName()
     {return currentMethodName;}

   protected Production listChildProductions()
     {
	int i=0,nr=0;
  	ListProduction ret=null;
  	if (children==null) return null;
//	System.out.println("Listing child productions of " + this);
	for(i=0;i<children.length;i=i+1)
	  {
	     SimpleNode s=(SimpleNode)children[i];
//	     System.out.println(" --> "+s);
	     Production r=s.generateProductions();
	     if (r!=null)
	       {
		  nr=nr+1;
		  ret=new ListProduction(r,ret);
	       }
	  }
	if (nr==1) return ret.getContent();
	return ret;
     }
   
   protected Production sequenceChildProductions()
     {
	int i=0;
  	Production ret=null;
  	if (children==null) return null;
	for(i=0;i<children.length;i=i+1)
	  ret=Production.sequence(ret,generateProductions(i));
	return ret;
     }
   
   protected Production createAsyncTerm()
     {
	ASTArgumentList s;
	String invocation;
	return new Term("*",retrieveInvoke());
     }
   protected Production sequenceLazies()
     {
	int i;
	Production last=generateProductions(0), ors=last;
	for(i=1;i<children.length;i++)
	  {
	     Production current=generateProductions(i);
	     if (current==null) continue;
	     last=Production.deep(last);
	     last=Production.sequence(last,current);
	     ors=Production.choice(last,ors);
	  }
	return ors;
     }
}

