package component.system;
import java.util.*;

abstract public class AbstractPort 
  extends Global
{
   protected Component owner;           // the owner of the port
   protected String name;               // an indicative name of the port
   
   public AbstractPort(String n, Component o)
     {
	owner=o;
	name=n;
	if (owner==null)
	  Fatal("abstractport: owner unknown");
	if (name==null)
	  Fatal("abstractport: name unknown");
	if (name.compareTo("")==0)
	  Fatal("abstractport: name empty");
     }
   
   public String getName()
     {
	return name;
     }
   
   public Component getOwner()
     {
	return owner;
     }
   
   public void setOwner(Component c)
     {
	owner=c;
     }
   
   public String exportId()
     {
	return owner.name+"/"+name;
     }
   
   public String toString()
     {
	return exportId();
     }
   
   /**
    * state checks
    */
   abstract boolean connected();
   abstract void unlink();
   abstract void relink(Component owner);
   
   /**
    * sending of a message is done in 3 steps
    * 1- somebody asks the port to send a message
    * 2- the port asks the component system if it is okay to send the message
    * 3- the port sends the message
    */
   public void sendMessage(Message m)
     {
	owner.filterSendMessage(this,m);
     }
   abstract public void doSendMessage(Message m);
   
   
   /**
    * receiving a message is done in 3 stepts
    * 1- somebody asks the port to receive a message
    * 2- the port asks his owner to receive the message
    * 3- the owner calls doReceive
    */
   public void receiveMessage(Message m)
     {
	owner.filterReceiveMessage(this,m);
     }
   abstract public void doReceiveMessage(Message m);
   
   /**
    * When we need to pass a creation procedure we can use this method
    * to specify a creator
    */
   static public AbstractPortCreator getCreator()
     {
	Fatal("abstractport: getcreator() called");
	return null;
     };
}

