package component.system;
import java.util.*;

/**
 * A directy port is a local direct link which connects to another port 
 * without passing trough the component system.
 */
public class DirectPort
  extends Port
{
   private AbstractPort targetport;       // the immediate target when sending messages

   /**
    * Creation 
    */
   public DirectPort(String type, Component owner)
     {
	super(type,owner);
     }
   public DirectPort(String type, Component owner, Component handler)
     {
	super(type,owner, new InvocationMessageHandler(handler));
     }
   public DirectPort(String type, Component owner, MessageHandler handler)
     {
	super(type,owner,handler);
     }   
   static public AbstractPortCreator getCreator()
     {
	return new DirectPortCreator();
     }
   
   /**
    * connecting to a port.
    * If connecting. connect(Port p) is called upon handleby
    * when disconnecting. disconnect(Port p) is called upon handleby
    */
   public void setTarget(PortAddress t)
     {
	// als de directport een port assigned krijgt, zoeken we 
	// de expliciete target op door het componentsysteem te vragen
	super.setTarget(t);
	setTargetPort(ComponentSystem.lookupDirectPort(t));
     }
   public void setTargetPort(AbstractPort end)
     {
	targetport=end;
     }

   /**
    * sending of a message is done in 3 steps
    * 1- somebody asks the port to send a message
    * 2- the port asks the component system if it is okay to send the message
    * 3- the port sends the message
    */
   public void doSendMessage(Message m)
     {
	// check target or owner
	if (targetport==null)
	  Fatal("directport: targetport==null");
	targetport.receiveMessage(m);
     }
   
   public boolean connected()
     {
	// connected for a direct port means that both sides of the 
	// port know each other. 
	return (target!=null) && (((DirectPort)targetport).target!=null);
     }
}

/**
 * The creator of direct ports 
 */
class DirectPortCreator extends AbstractPortCreator
{
   public AbstractPort createPort(String name, Component owner, MessageHandler mh)
     {
	return new DirectPort(name,owner,mh);
     }
}
