package component.system;
import component.system.util.*;
import java.lang.reflect.*;
import java.io.*;
import java.util.*;

/**
 * A message are the eventes which are send between components
 * Messages are cloned when sending from one component to 
 * another.
 */
public class Message
  extends Global
  implements Serializable
{
   // These are fields that store information about the message
   // standard values are :
   //    Invoke  : message to invoke
   //    From    : nbecessary for god knows what reason
   //    To      : nbecessary for god knows what reason
   
   private UtilHashMap rofields;
   private UtilHashMap rohidden;
   private UtilHashMap fields;
   private UtilHashMap hidden;
   private PortAddress target=null;
   
   synchronized final private void dumpfields(UtilHashMap h,String pre, String post)
     {
	if (h==null) 
	  return;
	HashMapIterator it = h.keySetIterator();
	while(it.hasNext())
	  {
	     String field = (String)it.next();
	     String w=field+" : "+h.get(field);
	     System.out.println(pre+w+post);
	  }
     }
    
   synchronized final public void dump()
     {
	dumpfields(fields,"<",">");
	dumpfields(hidden,">","<");
	dumpfields(rofields,"<<",">>");
	dumpfields(rohidden,">>","<<");
     }
   
   synchronized final public Object getField(String fieldName)
     {
	if (fields!=null && fields.containsKey(fieldName))
	  return fields.get(fieldName);
	else if (rofields!=null)
	  return rofields.get(fieldName);
	else 
	  return null;
     }
   
   synchronized final public Object getHiddenField(String fieldName)
     {
	if (hidden != null && hidden.containsKey(fieldName))
	  return hidden.get(fieldName);
	else if (rohidden!=null)
	  return rohidden.get(fieldName);
	else 
	  return null;
     }

   synchronized final public Object putField(String fieldName, Object o)
     {
	// System.out.println("Putting field "+(String)fieldName);
	if (fields==null)
	  fields=new UtilHashMap();
	fields.put(fieldName, o);
	return null;
     }
   
   synchronized final public Object putHiddenField(String fieldName, Object o)
     {
	// System.out.println("Putting hidden field "+(String)fieldName);
	if (hidden==null)
	  hidden=new UtilHashMap();
	hidden.put(fieldName,o);
	return null;
     }
   
   synchronized final protected void makeHiddenReadOnly()
     {
	if (rohidden==null)
	  {
	     rohidden = hidden;
	     hidden = null;
	  }
	else if (hidden!=null)
	  {
	     // first we copy the entire existing table
	     rohidden=new UtilHashMap(rohidden);
	     // then we copy the current hidden fields
	     rohidden.putAll(hidden);
	     // and finally we create a new set of hidden fields
	     hidden=new UtilHashMap();
	  }
     }
   synchronized final protected void makeFieldsReadOnly()
     {
	if (rofields==null)
	  {
	     rofields = fields;
	     fields = null;
	  }
	else if (fields!=null)
	  {
	     rofields = new UtilHashMap(rofields);
	     rofields.putAll(fields);
	     fields = new UtilHashMap();
	  }
     }
   
   public Message()
     { 
	// an original message is empty, with place for
	// writeable fields
	rohidden = null;
	rofields = null;
	hidden = null;
	fields = null;
     }

   public Message(Message m)
     {
	// if we make a copy of another message we need to make it 
	// first readonly
	
//	System.out.println("Msg...");
//	m.dump();

	m.makeHiddenReadOnly();
	// then we can share the fields
	rohidden = m.rohidden;
	rofields = null;
	hidden = null;
	fields = null;
     }
   

   public Message(Message m1, Message m2)
     {
	m1.makeHiddenReadOnly();
	m2.makeFieldsReadOnly();
	// then we can share the fields
	rohidden = m1.rohidden;
	rofields = m2.rofields;
	hidden = null;
	fields = null;
     }

   synchronized final public Message copy()
     {
	return new Message(this,this);
     }   

   synchronized final public PortAddress getTarget()
     {
	return target;
     }
   
   synchronized final public void setTarget(PortAddress t)
     {
	target = t;
     }

   synchronized public String toString () 
     {
	StringBuffer sb = new StringBuffer ();
	sb.append (getField ("Invoke"));
	return sb.toString (); 
     }
}
