package component.system;
import component.system.util.*;

public class MultiPort 
  extends AbstractMultiPort
{
   protected UtilVector ports;
   
   public MultiPort(String type, Component owner, MessageHandlerCreator m, AbstractPortCreator pc)
     {
	super(type,owner,m,pc);
	ports = new UtilVector();
     }
   
   public MultiPort(String type, Component owner, MessageHandler m, AbstractPortCreator pc)
     {
	this(type,owner,new SameHandlerCreator(m),pc);
     }
   
   public MultiPort(String type, Component owner, MessageHandlerCreator m)
     {
	this(type,owner,m,Port.getCreator());
     }

   public MultiPort(String type, Component owner, MessageHandler m)
     {
	this(type,owner,m,Port.getCreator());
     }

   public MultiPort(String type, Component owner, AbstractPortCreator pc)
     {
	this(type,owner, new SameHandlerCreator(owner),pc);
     }

   public MultiPort(String type, Component owner)
     {
	this(type,owner, new SameHandlerCreator(owner));
     }
   
   public void doSendMessage(Message m)
     {
	VectorIterator it=ports.iterator();
	while(it.hasNext())
	  {
	     Port port;
	     port=(Port)it.next();
	     if (port.connected())
	       port.sendMessage(m.copy());
	  }
     }

   public int connectionCount()
     {
	int count=0;
	VectorIterator it=ports.iterator();
	while(it.hasNext())
	  {
	     Port port;
	     port=(Port)it.next();
	     if (port.connected()) 
	       count++;
	  }
	return count;
     }
   
   public AbstractPort newPort()
     {
	AbstractPort port;
	port=getNewPort();
	ports.add(port);
	return port;
     }
   
   public UtilIterator iterator()
     {
	return ports.iterator();
     }
   
   public boolean contains(AbstractPort port)
     {
	VectorIterator it=ports.iterator();
	while(it.hasNext())
	  {
	     Port p=(Port)it.next();
	     if (p==port)
	       return true;
	  }
	return false;
     }
   public Port findPortWithTargetType(String type)
     {
	VectorIterator it=ports.iterator();
	while(it.hasNext())
	  {
	     Port p=(Port)it.next();
	     if (p.getInformation()==null)
	       continue;
	     if (p.getInformation().compareTo(type)==0)
	       return p;
	  }
	return null;
     }
   public boolean connected()
     {
	VectorIterator it=ports.iterator();
	while(it.hasNext())
	  {
	     Port port = (Port) it.next();
	     if (port.connected())
	       return true;
	  }
	return false;
     }
   
   protected void unlink()
     {
	VectorIterator it = ports.iterator();
	while(it.hasNext())
	  {
	     Port port = (Port)it.next();
	     port.unlink();
	  }
	owner=null;
     }
   protected void relink(Component t)
     {
	VectorIterator it = ports.iterator();
	while(it.hasNext())
	  {
	     Port port = (Port)it.next();
	     port.relink(t);
	  }
	owner=t;
     }
}
