package component.system;
import java.util.*;

/**
 * A port is an entry point or exit point for an object. It is used from within the component 
 * to send out messages, and to receive messages from other components.
 * A port has a unique ID, which, once addigned, never changes. 
 * This id is given by the component system. 
 * A port doesn't know with whom it is connected. 
 * A port knows its own type.
 * A port can map incoming messages to certain objects.
 */
public class Port
{
   private MessageHandler maponto;  // incoming messages will be mapped onto
   private String         id;       // the identification number of this port
   private Component      owner;    // the owner of the port
   private String         target;   // the target when sending messages
   public Port(Component o)
     {
	owner=o;
	ComponentSystem.announcePort(this);
     }
   public Port(Component o, MessageHandler h)
     {
	owner=o;
	maponto=h;
	ComponentSystem.announcePort(this);
     }
   public void mapOnto(MessageHandler o)
     {
	maponto=o;
     }
   public void setId(String i)
     {
	id=i;
     }   
   public String getId()
     {
	return id;
     }   
   public void setTarget(String t)
     {
	target=t;
     }
   public String getTarget()
     {
	return target;
     }
   
   
   /**
    * sending of a message is done in 3 steps
    * 1- somebody asks the port to send a message
    * 2- the port asks the component system if it is okay to send the message
    * 3- the port sends the message
    */
   public void sendMessage(Message m)
     {
	owner.filterSendMessage(this,m);
     }
   
   public void doSendMessage(Message m)
     {
	ComponentSystem.SendMessage(this,m);
     }
   
   
   /**
    * receiving a message is done in 3 stepts
    * 1- somebody asks the port to receive a message
    * 2- the port asks his owner to receive the message
    * 3- the owner calls doReceive
    */
   public void receiveMessage(Message m)
     {
	// when receiving a message, we execute the message upon maponto
	// FIXME: currently we simply call handleMessage
	owner.filterReceiveMessage(this,m);
     }
   public void doReceiveMessage(Message m)
     {
	// when receiving a message, we execute the message upon maponto
	// FIXME: currently we simply call handleMessage
	maponto.handleMessage(m);
     }
}
