package component.system;
import java.io.*;

public class StandardScheduler
  extends Thread
  implements Scheduler
{
   /**
    * inQ zijn al de berichten die binnengekomen zijn bij het verwerken
    * van de huidige thunk (of thunks). Berichten verhuizen automatisch van
    * de inQ naar 1 van de outQ's. Een outQ is een Q die een bericht
    * klaarzet bij een bepaalde thread. ThreadCount is het aantal threads
    * bruikbaar om ComponentMessages af te handelen.
    */
   protected static MessageQueue inQ = new MessageQueue();
   protected static MessageQueue[] outQs = null;
   protected static int threadCount=0;


   public StandardScheduler() {}
   
   public StandardScheduler(int tC)
     {
	initialise(tC);
     }

   public void initialise(int tC) 
     {
	int i;
	threadCount=tC;
	/* Q's initialiseren */
	outQs=new MessageQueue[threadCount];
	for(i=0;i<threadCount;i++)
	  {
	     outQs[i]=new MessageQueue();
	     new ExecutionLoop(outQs[i]);
	  }
        System.out.println(this+" initialised");
     }
   
   public void scheduleForExecution(Message m)
     {
	inQ.push(m);
     }
   
   /**
    * This is the loop that processes all incoming messages and keeps the
    * main-thread running.
    *
    * OPTIMALISATIE: deze methode moet er uit geknald worden als we opgeven
    * 		1 thread dan moeten we niet beginnen met messages heen en weer de
    *		jossen maar kunnen we ons maar beter bezig houden met onmiddelijk
    *		de juiste methode op te roepen.
    * NUANCERING: Dat is natuurlijk niet helemaal juist daar we dan recursieve
    *           calls naar dezelfde component kunnne hebben, wat ook niet 
    *           de bedoeling is.
    */
   public void run()
     {
	int currentout=0, emptyQnr,i;
	while(true)
	  {
	     Message m=(Message)inQ.pop();
	     /* zoek de outQ die bezig of zich bezig gaat houden
	      * met target */
	     PortAddress target = m.getTarget();
	     for(i=0, emptyQnr=-1; i<threadCount;i++)
	       {

		/* is this synchronization really needed? */
		  synchronized(outQs[i])
		    {
		       if (outQs[i].containsMessageFor(target))
			 {
			    outQs[i].push(m);
			    break;
			 }
		       if (outQs[i].isEmpty())
			 {
			    emptyQnr=i;
			 }
		    }
	       }
	     if (i<threadCount) continue;
	     /* indien niemand er zich gaat mee bezig houden of er mee bezig is
	      * kiezen we iemand die niets te doen heeft
	      */
	     if (emptyQnr>=0)
	       {
		  outQs[emptyQnr].push(m);
		  continue;
	       }
	     /* als er niemand niets te doen heeft pakken we maar iemand
	      */
	     outQs[currentout++].push(m);
	     if (currentout==threadCount) 
	       currentout=0;
	  }
     }
   
   public String toString()
     {
	return "Standard Scheduler [" + threadCount + "]";
     }
}
